import {LocalModels} from "./LocalModels.ts";
import {Configs} from "./customer/Configs.ts";
import {
    BaZi,
    DaYunItem,
    GaoNeng,
    LiuNian,
    LiuRi,
    LiuYue,
    QuShiInfo,
    SiZhu,
    UserInfo
} from "./customer/SupportModels.ts";
import {ref} from "vue";
import {QuShiLogic} from "./customer/QuShiLogic.ts";
import {RiYunShiLogic} from "./customer/RiYunShiLogic.ts";

export namespace DataCenter {

    export let wxOpenId = null

    export let shengShi: Map<string, string[]> = new Map()

    export let shiQu: Map<string, string[]> = new Map()

    export let quData: Map<string, LocalModels.Area> = new Map()

    export let nongliMonths: Map<number, string[]> = new Map() //指定年有哪些农历月

    export let nongliDays: Map<string, string> = new Map() //指定年月最大的农历日是什么

    export let bazi = ref<BaZi>()

    export let powerMode = ref(1)

    export let dayunIndex = ref()

    export let curDaYun = ref<DaYunItem>()

    export let liunianIndex = ref()

    export let curLiuNianList = ref([])

    export let curLiuNian = ref<LiuNian>()

    export let liuyueIndex = ref()

    export let curLiuYue = ref<LiuYue>()

    export let liuriIndex = ref()

    export let curLiuRi = ref<LiuRi>()

    export let includeDaYunInGeJu = ref(false)

    export let includeLiuNianInGeJu = ref(false)

    export let selectedDaYunInGeJu = ref()

    export let selectedLiuNianInGeJu = ref()

    export let lastKeyInGeJu = ref(null)

    export let owernId = 99

    export let ownerUsers = ref<UserInfo[]>([])

    export let historyUser = ref<UserInfo[]>([])

    export let historyBaZi = ref<BaZi[]>([])

    export function addAreaData(data: LocalModels.Area) {
        let key1 = data.sheng
        if (!shengShi.has(key1)) {
            shengShi.set(key1, [])
        }
        let lst1 = shengShi.get(key1)
        if (lst1.indexOf(data.shi) == -1) {
            lst1.push(data.shi)
        }

        let key2 = data.sheng + data.shi
        if (!shiQu.has(key2)) {
            shiQu.set(key2, [])
        }
        let lst2 = shiQu.get(key2)
        if (lst2.indexOf(data.qu) == -1) {
            lst2.push(data.qu)
        }

        let key3 = data.sheng + data.shi + data.qu
        quData.set(key3, data)
    }

    export function addNongLi(data: LocalModels.NongLi) {
        let months = nongliMonths.get(data.year)
        if (months == null) {
            months = []
            nongliMonths.set(data.year, months)
        }
        if (months.indexOf(data.month) == -1) {
            months.push(data.month)
        }
        let dayKey = data.year.toString() + data.month
        nongliDays.set(dayKey, data.day)
    }

    export function getNontLiMonths(year: number) {
        return nongliMonths.get(year)
    }

    export function getNongLiDays(year: number, month: string) {
        let result = []
        let key = year.toString() + month
        let maxDay = nongliDays.get(key)
        for (let i = 0; i < Configs.nongliAry.length; i++) {
            let nl = Configs.nongliAry[i]
            result.push(nl)
            if (nl == maxDay) {
                break
            }
        }
        return result
    }

    export function fixPinYin(key: string) {
        let result = []
        if (key.length > 0) {
            quData.forEach(function (value) {
                let allKey = value.shiPinYin + value.quPinYin
                if (allKey.indexOf(key) != -1) {
                    result.push(value)
                }
                let allName = value.sheng + value.shi + value.qu
                if (allName.indexOf(key) != -1) {
                    result.push(value)
                }
            })
        }

        return result
    }

    function calcFangXiang(gn: GaoNeng, zhu: SiZhu) {
        if (["合", "暗合", "半合"].indexOf(gn.guanxi) != -1) {
            if (zhu.text == gn.fangxiang) {
                gn.sign = ""
            } else {
                if (gn.wei_zhi3 == null) {
                    if (zhu.wei_zhi < gn.wei_zhi2) {
                        gn.sign = "→"
                    } else {
                        gn.sign = "←"
                    }
                } else {
                    let wz = gn.wei_zhi2
                    if (gn.fangxiang == gn.arg3) {
                        wz = gn.wei_zhi3
                    }
                    if (zhu.wei_zhi < wz) {
                        gn.sign = "→"
                    } else {
                        gn.sign = "←"
                    }
                }
            }
        }
    }

    function buildGaoNeng(riyuan: string, data: object, zhu1: SiZhu, zhu2: SiZhu, zhu3: SiZhu, trigger: string = null) {
        let gn = new GaoNeng()
        gn.arg1 = zhu1.text
        gn.arg2 = zhu2.text
        gn.wei_zhi1 = zhu1.wei_zhi
        gn.wei_zhi2 = zhu2.wei_zhi
        gn.wu_xing1 = zhu1.wu_xing
        gn.wu_xing2 = zhu2.wu_xing
        gn.shi_shen1 = zhu1.shi_shen
        gn.shi_shen2 = zhu2.shi_shen
        if (zhu3 != null) {
            gn.arg3 = zhu3.text
            gn.wei_zhi3 = zhu3.wei_zhi
            gn.wu_xing3 = zhu3.wu_xing
            gn.shi_shen3 = zhu3.shi_shen
        }
        gn.text = data["name"]
        gn.guanxi = data["guanxi"]
        gn.leibie = data["leibie"]
        gn.hehua = data["hehua"]
        gn.fangxiang = data["fangxiang"]
        if (["半合", "拱合", "三会"].indexOf(gn.guanxi) != -1) {
            gn.zengqiang_shishen = Configs.config_shishen[riyuan][gn.fangxiang]
        }

        if (gn.guanxi == "刑") {
            let xing = Configs.config_gaoneng_xing[gn.text]
            if (xing != null) {
                let chus = xing["刑出"]
                let chongs = xing["刑冲"]
                gn.xing_chu = ""
                gn.xing_chong = ""
                chus.forEach(function (chu) {
                    let xing_shi_shen = Configs.config_shishen[riyuan][chu]
                    gn.xing_chu += xing_shi_shen
                })
                chongs.forEach(function (chong) {
                    let chong_shi_shen1 = Configs.config_shishen[riyuan][chong[0]]
                    let chong_shi_shen2 = Configs.config_shishen[riyuan][chong[1]]
                    gn.xing_chong += chong_shi_shen1 + chong_shi_shen2 + "冲"
                })
            }
        }

        gn.tip = data["tip"]
        calcFangXiang(gn, zhu1)
        if (trigger == null) {
            zhu1.gaonengs.push(gn)
        } else {
            if (!zhu1.trigger_gaonengs.has(trigger)) {
                zhu1.trigger_gaonengs.set(trigger, [])
            }
            let lst = zhu1.trigger_gaonengs.get(trigger)
            lst.push(gn)
            //console.log(trigger, zhu1.trigger_gaonengs, gn.text, zhu1.wei_zhi, zhu1.arg1)
        }
    }

    export function fixGaoNengByZhu(riyuan: string, zhu1: SiZhu, zhu2: SiZhu, trigger: string) {
        if (zhu1 == zhu2) {
            return
        }
        let key = zhu1.text + zhu2.text
        let query = Configs.config_gaoneng_query[key]
        if (query != null) {
            query.forEach(function (gaonengData) {
                buildGaoNeng(riyuan, gaonengData, zhu1, zhu2, null, trigger)
                buildGaoNeng(riyuan, gaonengData, zhu2, zhu1, null, trigger)
            })
        }
    }

    export function fixGaoNeng(riyuan: string, zhu: SiZhu, targets: SiZhu[], trigger: string) {
        targets.forEach(function (sz) {
            if (zhu != sz) {
                let zhu1 = zhu
                let zhu2 = sz
                let key = zhu1.text + zhu2.text
                let query = Configs.config_gaoneng_query[key]
                if (query != null) {
                    query.forEach(function (gaonengData) {
                        buildGaoNeng(riyuan, gaonengData, zhu1, zhu2, null, null)
                        buildGaoNeng(riyuan, gaonengData, zhu2, zhu1, null, trigger)
                    })
                }
            }
            //Configs.config_gaoneng_query.__contains__(key):
        })
    }

    export function calcLiuYueGaoNeng(liuyue: LiuYue, dayun: DaYunItem, liunian: LiuNian) {
        if (DataCenter.bazi.value == null) {
            return
        }
        if (!liuyue.is_gaoneng_calced) {
            let riyuan = DataCenter.bazi.value.ri_gan.text
            let trigger = liuyue.trigger
            let tgs = [
                DataCenter.bazi.value.nian_gan,
                DataCenter.bazi.value.yue_gan,
                DataCenter.bazi.value.ri_gan,
                DataCenter.bazi.value.shi_gan,
                dayun.gan_zhu,
                liunian.gan_zhu,
            ]
            let dzs = [
                DataCenter.bazi.value.nian_zhi,
                DataCenter.bazi.value.yue_zhi,
                DataCenter.bazi.value.ri_zhi,
                DataCenter.bazi.value.shi_zhi,
                dayun.zhi_zhu,
                liunian.zhi_zhu,
            ]
            DataCenter.fixGaoNeng(riyuan, liuyue.gan_zhu, tgs, trigger)
            DataCenter.fixGaoNeng(riyuan, liuyue.zhi_zhu, dzs, trigger)
            liuyue.is_gaoneng_calced = true
        }
    }

    export function calcLiuRiGaoNeng(liuri: LiuRi, liuyue: LiuYue, dayun: DaYunItem, liunian: LiuNian) {
        if (DataCenter.bazi.value == null) {
            return
        }
        if (!liuri.is_gaoneng_calced) {
            let riyuan = DataCenter.bazi.value.ri_gan.text
            let trigger = liuri.trigger
            let tgs = [
                DataCenter.bazi.value.nian_gan,
                DataCenter.bazi.value.yue_gan,
                DataCenter.bazi.value.ri_gan,
                DataCenter.bazi.value.shi_gan,
                dayun.gan_zhu,
                liunian.gan_zhu,
                liuyue.gan_zhu,
            ]
            let dzs = [
                DataCenter.bazi.value.nian_zhi,
                DataCenter.bazi.value.yue_zhi,
                DataCenter.bazi.value.ri_zhi,
                DataCenter.bazi.value.shi_zhi,
                dayun.zhi_zhu,
                liunian.zhi_zhu,
                liuyue.zhi_zhu,
            ]
            DataCenter.fixGaoNeng(riyuan, liuri.gan_zhu, tgs, trigger)
            DataCenter.fixGaoNeng(riyuan, liuri.zhi_zhu, dzs, trigger)
            liuri.is_gaoneng_calced = true
        }
    }

    export function calcQuShi() {
        if (DataCenter.bazi.value == null) {
            return
        }
        DataCenter.bazi.value.qushiInfo = []
        //只计算未来三年的趋势，可以看到月份数据
        //大运、年、月趋势都只算自身的，年趋势数据不再需要月数据累加
        let len = 3
        let year = new Date().getFullYear()
        let data = QuShiLogic.findDaYunLiuNian(year, DataCenter.bazi.value)
        let dy = data[0]
        let ln = data[1]

        if (dy != null && ln != null) {
            let lushen = DataCenter.bazi.value.shishen_guanxi.get("禄神")[0]
            let banlu = DataCenter.bazi.value.shishen_guanxi.get("半禄")
            let dayunList: DaYunItem[] = [dy]
            let liunianList: LiuNian[] = [ln]
            while (liunianList.length < len) {
                let dt = QuShiLogic.getNext(DataCenter.bazi.value, dy, ln)
                if (dt == null) {
                    break
                } else {
                    dy = dt[0]
                    ln = dt[1]
                    dayunList.push(dy)
                    liunianList.push(ln)
                }
            }

            for (let i = 0; i < liunianList.length; i++) {
                let dayun = dayunList[i]
                let liunian = liunianList[i]
                let yunKey = dayun.gan_text + dayun.zhi_text + "运"
                let nianKey = liunian.year.toString() + "年"

                let info = new QuShiInfo()
                info.year = liunian.year

                QuShiLogic.calcQuShiScore(dayun.gan_zhu, info, lushen, banlu, yunKey, 0)
                QuShiLogic.calcQuShiScore(dayun.zhi_zhu, info, lushen, banlu, yunKey, 0)
                QuShiLogic.calcQuShiScore(liunian.gan_zhu, info, lushen, banlu, nianKey, 1)
                QuShiLogic.calcQuShiScore(liunian.zhi_zhu, info, lushen, banlu, nianKey, 1)

                for (let j = 0; j < liunian.liuyue.length; j++) {
                    let liuyue = liunian.liuyue[j]
                    let yueKey = nianKey + liuyue.month.toString() + "月"
                    if (!liuyue.is_gaoneng_calced) {
                        calcLiuYueGaoNeng(liuyue, dayun, liunian)
                    }
                    QuShiLogic.calcQuShiScore(liuyue.gan_zhu, info, lushen, banlu, yueKey, 2, liuyue.month)
                    QuShiLogic.calcQuShiScore(liuyue.zhi_zhu, info, lushen, banlu, yueKey, 2, liuyue.month)
                }

                info.calcTotal()
                DataCenter.bazi.value.qushiInfo.push(info)
            }
        }

    }

    export function calcRiYunShi() {
        if (DataCenter.bazi.value != null) {
            DataCenter.bazi.value.dailyQuShi = []
            DataCenter.bazi.value.dailyQuShiDate = []
            for (let i = 0; i <= 1; i++) {
                let date = new Date()
                //date.setDate(31)
                date.setDate(date.getDate() + i)
                let year = date.getFullYear()
                let month = date.getMonth() + 1
                let day = date.getDate()
                let data = RiYunShiLogic.findNianYueRi(year, month, day, DataCenter.bazi.value)
                let ln = data[0]
                let ly = data[1]
                let lr = data[2]
                if (ln != null && ly != null && lr != null) {
                    let map = RiYunShiLogic.calc(ln, ly, lr)
                    DataCenter.bazi.value.dailyQuShi.push(map)
                    DataCenter.bazi.value.dailyQuShiDate.push(date.getFullYear() + "-" + (date.getMonth() + 1) + "-" + date.getDate() + " " + lr.gan_text + lr.zhi_text + "日")
                }
            }
        }
    }
}