import {LocalModels} from "./LocalModels.ts";
import {Tools} from "./Tools.ts";

export namespace Network {

    let sendingTime: Map<string, number> = new Map()

    let debugMode = false

    export function applyUserInfo(request: LocalModels.UserRequest, cb: (rsp: LocalModels.BaseResponse) => void) {
        post(getServerUrl() + "applyUserInfo", request.toData(), function (response: string) {
            let dt = LocalModels.BaseResponse.build(response)
            cb && cb(dt)
        })
    }

    function getServerUrl() {
        if (!debugMode) {
            return "https://yixuefrp.cxhy.cn/wx/"
        } else {
            return "http://127.0.0.1:13008/wx/"
        }
    }

    export function post(url: string, data: object, cb: (response: string) => void) {
        let tm = Date.now()
        let allow: boolean = false
        if (!sendingTime.has(url)) {
            sendingTime.set(url, tm)
            allow = true
        } else {
            let last = sendingTime.get(url)

            if (tm - last > 1000) {
                sendingTime.set(url, tm)
                allow = true
            }
        }
        if (!allow) {
            cb && cb(null)
            Tools.showMessage("操作太快了，请慢一点")
            return
        }

        let requst = new XMLHttpRequest()
        requst.open("POST", url, true)
        requst.setRequestHeader('Content-Type', 'application/json');
        //requst.setRequestHeader('Access-Control-Allow-Origin', '*');
        let dt = JSON.stringify(data)
        requst.onreadystatechange = function () {
            if (requst.readyState == XMLHttpRequest.DONE) {
                if (requst.status == 200) {
                    cb && cb(requst.responseText)
                } else {
                    console.error(requst.statusText)
                }
            }
        }
        requst.send(dt)
    }
}