import {
    AreaResult,
    BaZi,
    DeleteUserRequest,
    LoadUserRequest,
    LoginRequest,
    LoginResult,
    NongLiRequest,
    NongliResult,
    SiZhuCheckResult,
    SiZhuRequest,
    SiZhuResult,
    UserInfo,
    XinLiRequest,
    XinLiResult
} from "./SupportModels.ts";
import {Tools} from "../Tools.ts";
import {DataCenter} from "../DataCenter.ts";

export namespace APICenter {


    let sendingTime: Map<string, number> = new Map()

    export let debugMode = false

    function getDeviceFingerprint() {
        var fingerprint = {
            userAgent: navigator.userAgent,
            screenResolution: [screen.width, screen.height],
            colorDepth: screen.colorDepth,
            timezoneOffset: new Date().getTimezoneOffset(),
            language: navigator.language,
            //plugins: Array.from(navigator.plugins).map(plugin => plugin.name),
            //canvasFingerprint: getCanvasFingerprint()
        };
        return fingerprint;
    }

    export function login(account: string, password: string, cb: (result: LoginResult) => void) {
        let request = new LoginRequest()
        request.account = account
        request.password = password
        let finger = getDeviceFingerprint()
        request.userAgent = finger.userAgent.toString()
        request.screenResolution = finger.screenResolution.toString()
        request.colorDepth = finger.colorDepth.toString()
        request.timezoneOffset = finger.timezoneOffset.toString()
        request.language = finger.language.toString()

        post(getServerUrl() + "login", beanToObject(request), function (response: string) {
            try {
                let result = LoginResult.build(response)
                if (result.code > 0) {
                    Tools.setCookie("yx_uname", result.name)
                    Tools.setCookie("yx_token", result.token)
                    Tools.setCookie("yx_uid", result.uid)
                } else {
                    Tools.removeCookie("yx_uname")
                    Tools.removeCookie("yx_token")
                    Tools.removeCookie("yx_uid")
                }
                cb && cb(result)
            } catch (e) {
                console.error(e)
                cb && cb(null)
            }
        })
    }

    export function requestByXinLi(name: string, isMan: boolean, xinliResult: XinLiResult, areaResult: AreaResult, cb: (code: number) => void) {
        let token = Tools.getCookie("yx_token")
        if (token == null) {
            Tools.showMessage("用户信息有误，请重新登录后重试")
            return
        }
        let request = new XinLiRequest()
        request.token = token
        request.uid = Tools.getCookie("yx_uid")
        request.name = name
        request.isMan = isMan

        request.year = xinliResult.year
        request.month = xinliResult.month
        request.day = xinliResult.day
        request.hourMode = xinliResult.hourMode
        request.hour = xinliResult.hour
        request.minute = xinliResult.minute

        request.areaMode = areaResult.mode
        request.sheng = areaResult.sheng
        request.shi = areaResult.shi
        request.qu = areaResult.qu

        post(getServerUrl() + "getSiZhu", beanToObject(request), function (response: string) {
            try {
                let code = resolveResponse(response)
                cb && cb(code)
            } catch (e) {
                console.error(e)
                cb && cb(-200)
            }
        })
    }

    export function requestByNongLi(name: string, isMan: boolean, nongliResult: NongliResult, areaResult: AreaResult, cb: (code: number) => void) {
        let token = Tools.getCookie("yx_token")
        if (token == null) {
            Tools.showMessage("用户信息有误，请重新登录后重试")
            return
        }

        let request = new NongLiRequest()
        request.uid = Tools.getCookie("yx_uid")
        request.token = token
        request.name = name
        request.isMan = isMan

        request.year = nongliResult.year
        request.month = nongliResult.month
        request.day = nongliResult.day
        request.hourMode = nongliResult.hourMode
        request.hour = nongliResult.hour
        request.minute = nongliResult.minute

        request.areaMode = areaResult.mode
        request.sheng = areaResult.sheng
        request.shi = areaResult.shi
        request.qu = areaResult.qu

        post(getServerUrl() + "getSiZhuByNongLi", beanToObject(request), function (response: string) {
            try {
                let code = resolveResponse(response)
                cb && cb(code)
            } catch (e) {
                console.error(e)
                cb && cb(-200)
            }
        })
    }

    export function requestBySiZhu(name: string, isMan: boolean, sizhuResult: SiZhuResult, cb: (checker: SiZhuCheckResult, code: number) => void) {
        let token = Tools.getCookie("yx_token")
        if (token == null) {
            Tools.showMessage("用户信息有误，请重新登录后重试")
            return
        }

        let request = new SiZhuRequest()
        request.uid = Tools.getCookie("yx_uid")
        request.token = token
        request.name = name
        request.isMan = isMan
        request.niangan = sizhuResult.niangan
        request.nianzhi = sizhuResult.nianzhi
        request.yuegan = sizhuResult.yuegan
        request.yuezhi = sizhuResult.yuezhi
        request.rigan = sizhuResult.rigan
        request.rizhi = sizhuResult.rizhi
        request.shigan = sizhuResult.shigan
        request.shizhi = sizhuResult.shizhi
        request.hourMode = sizhuResult.hourMode
        request.ignoreError = sizhuResult.ignoreError

        post(getServerUrl() + "getSiZhuByTexts", beanToObject(request), function (response: string) {
            try {
                let obj = JSON.parse(response)
                if (obj["code"] < 0) {
                    let checker = SiZhuCheckResult.build(obj["data"])
                    cb && cb(checker, obj["code"])
                } else {
                    let code = resolveResponse(response)
                    cb && cb(null, code)
                }
            } catch (e) {
                console.error(e)
                cb && cb(null, -200)
            }
        })
    }

    export function saveUser(request: UserInfo, cb: () => void) {
        post(getServerUrl() + "saveUser", beanToObject(request), function (response: string) {
            try {
                let obj = JSON.parse(response)
                let ary = obj["data"]
                let temp = []
                ary.forEach(function (data) {
                    let info = new UserInfo()
                    fillDataFromObject(info, data)
                    temp.push(info)
                })
                DataCenter.ownerUsers.value = temp
                console.log("共获得", DataCenter.ownerUsers.value.length, "条数据")
                cb && cb()
            } catch (e) {
                console.error(e)
                cb && cb()
            }
        })
    }

    export function loadUsers(cb: () => void) {
        let request = new LoadUserRequest()
        request.owner_id = DataCenter.owernId
        post(getServerUrl() + "loadUser", beanToObject(request), function (response: string) {
            try {
                let obj = JSON.parse(response)
                let ary = obj["data"]
                let temp = []
                ary.forEach(function (data) {
                    let info = new UserInfo()
                    fillDataFromObject(info, data)
                    temp.push(info)
                })
                DataCenter.ownerUsers.value = temp
                console.log("共获得", DataCenter.ownerUsers.value.length, "条数据")
            } catch (e) {
                console.error(e)
            }
            cb && cb()
        })
    }

    export function deleteUser(id: number) {
        let request = new DeleteUserRequest()
        request.owner_id = DataCenter.owernId
        request.user_id = id
        post(getServerUrl() + "deleteUser", beanToObject(request), function (response: string) {
            try {
                let obj = JSON.parse(response)
                let ary = obj["data"]
                let temp = []
                ary.forEach(function (data) {
                    let info = new UserInfo()
                    fillDataFromObject(info, data)
                    temp.push(info)
                })
                DataCenter.ownerUsers.value = temp
                console.log("共获得", DataCenter.ownerUsers.value.length, "条数据")
            } catch (e) {
                console.error(e)
            }
        })
    }

    function fillDataFromObject(bean: any, obj: object) {
        Object.keys(obj).forEach(function (key) {
            bean[key] = obj[key]
        })
    }

    function resolveResponse(response: string) {
        let obj = JSON.parse(response)
        console.log("raw:", obj)
        if (obj["code"] == -100) {
            Tools.showMessage("token有误，请登出后重试")
            return -100
        }
        if (obj["code"] == -200) {
            return -200
        }
        let bazi = BaZi.build(obj["data"])
        DataCenter.bazi.value = bazi
        DataCenter.powerMode.value = 1
        DataCenter.dayunIndex.value = null
        DataCenter.curDaYun.value = null
        DataCenter.curLiuNianList.value = []
        DataCenter.liunianIndex.value = null
        DataCenter.curLiuNian.value = null
        DataCenter.liuyueIndex.value = null
        DataCenter.curLiuYue.value = null
        DataCenter.liuriIndex.value = null
        DataCenter.curLiuRi.value = null
        DataCenter.includeDaYunInGeJu.value = false
        DataCenter.includeLiuNianInGeJu.value = false
        DataCenter.selectedDaYunInGeJu.value = null
        DataCenter.selectedLiuNianInGeJu.value = null
        DataCenter.calcQuShi()
        DataCenter.calcRiYunShi()
        return 200
    }

    function getServerUrl() {
        if (!debugMode) {
            return "https://yixuedjango.cxhy.cn/wx/"
            //https://yixuedjango.cxhy.cn/
            //https://yixuefrp.cxhy.cn/wx/
        } else {
            return "http://127.0.0.1:13008/wx/"
        }
    }

    export function beanToObject(bean: object) {
        let result = {}
        Object.keys(bean).forEach(function (key) {
            result[key] = bean[key]
        })
        return result
    }

    function post(url: string, data: object, cb: (response: string) => void) {
        let tm = Date.now()
        let allow: boolean = false
        if (!sendingTime.has(url)) {
            sendingTime.set(url, tm)
            allow = true
        } else {
            let last = sendingTime.get(url)

            if (tm - last > 1000) {
                sendingTime.set(url, tm)
                allow = true
            }
        }
        if (!allow) {
            cb && cb(null)
            Tools.showMessage("操作太快了，请慢一点")
            return
        }

        let requst = new XMLHttpRequest()
        requst.open("POST", url, true)
        requst.setRequestHeader('Content-Type', 'application/json');
        //requst.setRequestHeader('Access-Control-Allow-Origin', '*');
        let dt = JSON.stringify(data)
        requst.onreadystatechange = function () {
            if (requst.readyState == XMLHttpRequest.DONE) {
                if (requst.status == 200) {
                    cb && cb(requst.responseText)
                } else {
                    console.error(requst.statusText)
                }
            }
        }
        requst.send(dt)
    }
}