//tsconfig.json 中，要开启：
// "experimentalDecorators": true,
// "emitDecoratorMetadata": true

export namespace DecoratorController {

    let propertyMap: Map<string, any[]> = new Map()

    //基础类型直接设置成@use(MyClass)，复合类型设置成(Array,MyClass)或者(Map,MyClass)
    export function use(cls: any, subCls: any = null, minCls: any = null) {
        return function (target: Object, propertyKey: string) {
            //Reflect.defineMetadata(METADATA_KEY, param, target, propertyKey);
            //console.log(target.constructor.name, propertyKey)
            let key = target.constructor.name + propertyKey
            propertyMap.set(key, [cls, subCls, minCls])
        };
    }

    function resolveAsMap(data: object, cls: any, cls2: any) {
        let target = new Map()
        Object.keys(data).forEach(function (key) {
            let value = data[key]
            if (cls == null) {
                target.set(key, value)
            } else if (cls == Array) {
                target.set(key, resolveAsArray(value, cls2))
            } else {
                let temp = new cls()
                resolve(temp, value)
                target.set(key, temp)
            }
        })
        return target
    }

    function resolveAsArray(data: object, cls: any) {
        let target = []
        if (Array.isArray(data)) {
            data.forEach(function (value) {
                if (cls == null) {
                    target.push(value)
                } else {
                    let temp = new cls()
                    resolve(temp, value)
                    target.push(temp)
                }

            })
        }
        return target
    }

    function getMember(ary: any[], index: number) {
        let result = null
        if (index < ary.length) {
            result = ary[index]
        }
        return result
    }

    export function resolve(bean: object, data: object) {
        let attrNames = Object.getOwnPropertyNames(bean)
        let clsName = bean.constructor.name
        Object.keys(data).forEach(function (attr) {
            let value = data[attr]
            if (bean instanceof Map) {
                bean.set(attr, value)
            } else if (attrNames.indexOf(attr) != -1) {
                let classes: any[] = propertyMap.get(clsName + attr)
                if (classes == null) {
                    //常规字段处理
                    bean[attr] = value
                } else {
                    if (getMember(classes, 0) == Array) {
                        //数组字段处理
                        bean[attr] = resolveAsArray(value, getMember(classes, 1))
                    } else if (getMember(classes, 0) == Map) {
                        //字典字段处理
                        bean[attr] = resolveAsMap(value, getMember(classes, 1), getMember(classes, 2))
                    } else {
                        //自定义类型处理
                        let cs = getMember(classes, 0)
                        if (cs != null) {
                            let target = new cs()
                            resolve(target, value)
                            bean[attr] = target
                        }
                    }
                }
            } else {
                console.error(clsName, "中没有字段：", attr)
            }
        })
    }
}