import {BaZi, DailyQuShi, DaYunItem, GaoNeng, LiuNian, QuShiInfo, SiZhu} from "./SupportModels.ts";
import {Configs} from "./Configs.ts";

export namespace QuShiLogic {
    function hasHuaSha(zhu: SiZhu) {
        let hasHe = false
        let hasSha = false
        for (let i = 0; i < zhu.gaonengs.length; i++) {
            let gn = zhu.gaonengs[i]
            if (["合", "暗合"].indexOf(gn.guanxi) != -1) {
                hasHe = true
            } else if (["冲", "刑", "穿", "破"].indexOf(gn.guanxi) != -1) {
                hasSha = true
            }
        }
        return hasHe && hasSha
    }

    function getOrderScore(gn: GaoNeng) {
        let score = 0
        if (["自刑", "三会"].indexOf(gn.guanxi) == -1) {
            score = Configs.config_gaoneng_score[gn.guanxi][gn.shi_shen1][gn.shi_shen2]
        }
        return score
    }

    function getGuanXiScore(gn: GaoNeng) {
        //默认0分，表示不发生高能关系时，不好也不坏,extra 常规下0，有六合化煞的时候，加分
        let score = Configs.config_gaoneng_guanxi_score[gn.guanxi]
        if (score == null) {
            score = 0
        }
        return score
    }

    function calcTotalScore(orderScore: number, guanxiScore: number, lushen: string, banlu: string[], gn: GaoNeng) {
        let total = orderScore + guanxiScore
        let param = 0
        if (gn.arg1 == lushen || gn.arg2 == lushen) {
            param = 1
        } else if (banlu.indexOf(gn.arg1) != -1 || banlu.indexOf(gn.arg2) != -1) {
            param = 0.5
        }

        if (param > 0) {
            let lushenScore = 0
            if (Configs.config_gaoneng_lushen_score[gn.guanxi] != null) {
                lushenScore = Configs.config_gaoneng_lushen_score[gn.guanxi]
                let eft = 1 + lushenScore * param / 100
                total = Math.round(eft * total)
            }
        }

        return total
    }

    //mode : 0 日 1 时
    export function calcSiZhuScore(zhu: SiZhu, daily: DailyQuShi, lushen: string, banlu: string[], title: string, mode: number, hour: number, excludesWeiZhi: number[]) {
        let extra = 0
        if (hasHuaSha(zhu)) {
            extra = 30
        }
        for (let i = 0; i < zhu.gaonengs.length; i++) {
            let gn = zhu.gaonengs[i]
            if (gn.text == "辰戌冲" || gn.text == "丑未冲") {
                continue
            }

            if (excludesWeiZhi.indexOf(gn.wei_zhi1) != -1 || excludesWeiZhi.indexOf(gn.wei_zhi2) != -1) {
                continue
            }

            //排序分，就是合什么，冲什么的不同分数
            let orderScore = getOrderScore(gn)
            let guanxiScore = getGuanXiScore(gn)
            if (gn.guanxi == "合" || gn.guanxi == "暗合") {
                guanxiScore += extra
            }
            let totalScore = calcTotalScore(orderScore, guanxiScore, lushen, banlu, gn)
            if (totalScore == 0) {
                continue
            }

            let reason = title + " : " + gn.shi_shen1 + gn.guanxi + gn.shi_shen2 + "(" + gn.text + ")" + "(" + Configs.nameOfZhu[gn.wei_zhi1] + "+" + Configs.nameOfZhu[gn.wei_zhi2] + ")" + " => " + totalScore.toString()
            let mapKey1 = gn.shi_shen1
            let mapKey2 = gn.shi_shen2
            if (mode == 1 && hour > 0) {
                mapKey1 = gn.shi_shen1 + "_" + hour.toString()
                mapKey2 = gn.shi_shen2 + "_" + hour.toString()
            }

            let raw1 = daily.getValue(mapKey1, mode)
            raw1 += totalScore
            daily.setValue(mapKey1, raw1, mode)
            daily.addDetail(mapKey1, reason, mode)

            let raw2 = daily.getValue(mapKey2, mode)
            raw2 += totalScore
            daily.setValue(mapKey2, raw2, mode)
            daily.addDetail(mapKey2, reason, mode)
        }
    }

    //mode: 0: 大运，1:流年 2:流月
    export function calcQuShiScore(zhu: SiZhu, info: QuShiInfo, lushen: string, banlu: string[], title: string, mode: number, month: number = 0) {
        let extra = 0
        if (hasHuaSha(zhu)) {
            extra = 30
        }
        for (let i = 0; i < zhu.gaonengs.length; i++) {
            let gn = zhu.gaonengs[i]
            if (gn.text == "辰戌冲" || gn.text == "丑未冲") {
                continue
            }
            //排序分，就是合什么，冲什么的不同分数
            let orderScore = getOrderScore(gn)
            let guanxiScore = getGuanXiScore(gn)
            if (gn.guanxi == "合" || gn.guanxi == "暗合") {
                guanxiScore += extra
            }
            let totalScore = calcTotalScore(orderScore, guanxiScore, lushen, banlu, gn)
            let reason = title + " : " + gn.shi_shen1 + gn.guanxi + gn.shi_shen2 + "(" + gn.text + ")" + "(" + Configs.nameOfZhu[gn.wei_zhi1] + "+" + Configs.nameOfZhu[gn.wei_zhi2] + ")" + " => " + totalScore.toString()
            let mapKey1 = gn.shi_shen1
            let mapKey2 = gn.shi_shen2
            if (mode == 2 && month > 0) {
                mapKey1 = gn.shi_shen1 + "_" + month.toString()
                mapKey2 = gn.shi_shen2 + "_" + month.toString()
            }

            let raw1 = info.getValue(mapKey1, mode)
            raw1 += totalScore
            info.setValue(mapKey1, raw1, mode)
            info.addDetail(mapKey1, reason, mode)

            let raw2 = info.getValue(mapKey2, mode)
            raw2 += totalScore
            info.setValue(mapKey2, raw2, mode)
            info.addDetail(mapKey2, reason, mode)

        }
    }

    export function findDaYunLiuNian(year: number, bazi: BaZi) {
        let targetDaYun = null
        let targetLiuNian = null
        for (let i = 0; i < bazi.dayun.length; i++) {
            let dy = bazi.dayun[i]
            for (let j = 0; j < dy.liunian.length; j++) {
                let ln = dy.liunian[j]
                if (ln.year == year) {
                    targetDaYun = dy
                    targetLiuNian = ln
                    break
                }
            }
        }
        return [targetDaYun, targetLiuNian]
    }

    export function getNext(bazi: BaZi, dayun: DaYunItem, liunian: LiuNian) {
        let dyIndex = bazi.dayun.indexOf(dayun)
        let lnIndex = dayun.liunian.indexOf(liunian)
        let targetDaYun: DaYunItem = null
        let targetLiuNian: LiuNian = null
        if (dyIndex != -1 && lnIndex != -1) {
            lnIndex++
            if (lnIndex < dayun.liunian.length) {
                targetDaYun = dayun
                targetLiuNian = dayun.liunian[lnIndex]
            } else {
                dyIndex++
                if (dyIndex < bazi.dayun.length) {
                    targetDaYun = bazi.dayun[dyIndex]
                    targetLiuNian = targetDaYun.liunian[0]
                }
            }
        }

        if (targetDaYun == null || targetLiuNian == null) {
            return null
        } else {
            return [targetDaYun, targetLiuNian]
        }
    }
}

