import {BaZi, DailyQuShi, LiuNian, LiuRi, LiuYue, SiZhu} from "./SupportModels.ts";
import {Configs} from "./Configs.ts";
import {DataCenter} from "../DataCenter.ts";
import {QuShiLogic} from "./QuShiLogic.ts";

export namespace RiYunShiLogic {


    let tianganTexts = ["甲", "乙", "丙", "丁", "戊", "己", "庚", "辛", "壬", "癸"]

    export function findNianYueRi(year: number, month: number, day: number, bazi: BaZi) {
        let targetLiuNian = null
        let targetLiuYue = null
        let targetRi = null
        for (let i = 0; i < bazi.dayun.length; i++) {
            let dy = bazi.dayun[i]
            let ln: LiuNian = dy.findLiuNian(year)
            let ly: LiuYue = ln?.findLiuYue(month)
            let lr: LiuRi = ly?.findLiuRi(day)
            if (ln != null && ly != null && lr != null) {
                targetLiuNian = ln
                targetLiuYue = ly
                targetRi = lr
                break
            }
        }
        return [targetLiuNian, targetLiuYue, targetRi]
    }

    function getTianGanStartHour(riGan: string) {
        if (riGan == "甲" || riGan == "己") {
            return "甲"
        }

        if (riGan == "乙" || riGan == "庚") {
            return "丙"
        }

        if (riGan == "丙" || riGan == "辛") {
            return "戊"
        }

        if (riGan == "丁" || riGan == "壬") {
            return "庚"
        }

        if (riGan == "戊" || riGan == "癸") {
            return "壬"
        }

    }

    function getDistanceOfHour(hour: number) {
        let dist = 0
        if (hour == 23 || hour == 0) {
            dist = 0
        } else if (hour == 1 || hour == 2) {
            dist = 1
        } else if (hour == 3 || hour == 4) {
            dist = 2
        } else if (hour == 5 || hour == 6) {
            dist = 3
        } else if (hour == 7 || hour == 8) {
            dist = 4
        } else if (hour == 9 || hour == 10) {
            dist = 5
        } else if (hour == 11 || hour == 12) {
            dist = 6
        } else if (hour == 13 || hour == 14) {
            dist = 7
        } else if (hour == 15 || hour == 16) {
            dist = 8
        } else if (hour == 17 || hour == 18) {
            dist = 9
        } else if (hour == 19 || hour == 20) {
            dist = 10
        } else if (hour == 21 || hour == 22) {
            dist = 11
        }
        return dist
    }

    function getDiZhiByHour(hour: number) {
        let dist = getDistanceOfHour(hour)
        return Configs.dizhiAry[dist]
    }

    function getNextByDist(ary: string[], item: string, dist: number) {
        let result = null
        if (ary.indexOf(item) != -1) {
            let index = ary.indexOf(item)
            index += dist
            let ln = ary.length
            while (index >= ln) {
                index -= ln
            }
            result = ary[index]
        }
        return result
    }

    function nextTianGan(tiangan: string, dist: number) {
        return getNextByDist(Configs.tianganAry, tiangan, dist)
    }

    function fixGaoNengs(riyuan: string, ary1: SiZhu[], ary2: SiZhu[]) {
        for (let i = 0; i < ary1.length; i++) {
            DataCenter.fixGaoNengByZhu(riyuan, ary1[i], ary2[i], null)
        }
    }


    export function calc(liunian: LiuNian, liuyue: LiuYue, liuri: LiuRi) {
        let lushen = DataCenter.bazi.value.shishen_guanxi.get("禄神")[0]
        let banlu = DataCenter.bazi.value.shishen_guanxi.get("半禄")
        let map: Map<string, DailyQuShi> = new Map()

        tianganTexts.forEach(function (tg) {
            let daily = new DailyQuShi()
            let main_gan = createZhu(tg, tg, Configs.weizhi_ri_gan)
            let nian_gan = createZhu(liunian.gan_text, tg, Configs.weizhi_liunian_gan)
            let nian_zhi = createZhu(liunian.zhi_text, tg, Configs.weizhi_liunian_zhi)
            let yue_gan = createZhu(liuyue.gan_text, tg, Configs.weizhi_liuyue_gan)
            let yue_zhi = createZhu(liuyue.zhi_text, tg, Configs.weizhi_liuyue_zhi)
            let ri_gan = createZhu(liuri.gan_text, tg, Configs.weizhi_liuri_gan)
            let ri_zhi = createZhu(liuri.zhi_text, tg, Configs.weizhi_liuri_zhi)
            let hours = [9, 11, 13, 15, 17, 19, 21]
            let hourTianGanStart = getTianGanStartHour(ri_gan.text)
            let dayKey = liuyue.month.toString() + "月" + liuri.day.toString() + "日"

            fixGaoNengs(main_gan.text,
                [ri_gan, ri_gan, ri_gan, yue_gan, yue_gan, nian_gan],
                [main_gan, nian_gan, yue_gan, main_gan, nian_gan, main_gan])

            fixGaoNengs(main_gan.text,
                [ri_zhi, ri_zhi, yue_zhi],
                [nian_zhi, yue_zhi, nian_zhi])


            hours.forEach(function (hour) {
                let hourKey = dayKey + " " + hour.toString() + "点"
                let dist = getDistanceOfHour(hour)
                let hourTianGan = nextTianGan(hourTianGanStart, dist)
                let hourDiZhi = getDiZhiByHour(hour)
                let shi_gan = createZhu(hourTianGan, tg, Configs.weizhi_liushi_gan)
                let shi_zhi = createZhu(hourDiZhi, tg, Configs.weizhi_liushi_zhi)

                fixGaoNengs(main_gan.text,
                    [shi_gan, shi_gan, shi_gan, shi_gan],
                    [main_gan, nian_gan, yue_gan, ri_gan])
                fixGaoNengs(main_gan.text,
                    [shi_zhi, shi_zhi, shi_zhi],
                    [nian_zhi, yue_zhi, ri_zhi])
                // console.log(tg, hour)
                // console.log("    ", main_gan, nian_gan, yue_gan, ri_gan, shi_gan)
                // console.log("    ", nian_zhi, yue_zhi, ri_zhi, shi_zhi)
                // console.log()

                QuShiLogic.calcSiZhuScore(shi_gan, daily, lushen, banlu, hourKey, 1, hour, [])
                QuShiLogic.calcSiZhuScore(shi_zhi, daily, lushen, banlu, hourKey, 1, hour, [])
            })
            QuShiLogic.calcSiZhuScore(ri_gan, daily, lushen, banlu, dayKey, 0, 0, [20])
            QuShiLogic.calcSiZhuScore(ri_zhi, daily, lushen, banlu, dayKey, 0, 0, [20])
            daily.calcTotal()
            map.set(tg, daily)
        })
        return map
    }

    function createZhu(text: string, riyuan: string, weizhi: number) {
        let sz = new SiZhu()
        sz.text = text
        sz.wu_xing = Configs.config_wuxing[text]
        sz.shi_shen = Configs.config_shishen[riyuan][text]
        sz.wei_zhi = weizhi
        sz.gaonengs = []
        return sz
    }
}