import {DecoratorController} from "./DecoratorController.ts";
import {DataCenter} from "../DataCenter.ts";
import resolve = DecoratorController.resolve;
import use = DecoratorController.use;


export class XinLiResult {
    year: number
    month: number
    day: number
    hourMode: number
    hour: number
    minute: number
}

export class AreaResult {
    mode: number
    sheng: string
    shi: string
    qu: string
}

export class SiZhuResult {
    niangan: string
    nianzhi: string
    yuegan: string
    yuezhi: string
    rigan: string
    rizhi: string
    shigan: string
    shizhi: string
    hourMode: number
    ignoreError: boolean
}

export class NongliResult {
    year: number
    month: string
    day: string
    hourMode: number
    hour: number
    minute: number
}

export class LoginRequest {
    account: string
    password: string
    userAgent: string
    screenResolution: string
    colorDepth: string
    timezoneOffset: string
    language: string
}

export class XinLiRequest {
    token: string
    uid: number
    name: string
    isMan: boolean
    year: number
    month: number
    day: number
    hourMode: number
    hour: number
    minute: number
    areaMode: number
    sheng: string
    shi: string
    qu: string
}

export class NongLiRequest {
    token: string
    uid: number
    name: string
    isMan: boolean
    year: number
    month: string
    day: string
    hourMode: number
    hour: number
    minute: number
    areaMode: number
    sheng: string
    shi: string
    qu: string
}

export class SiZhuRequest {
    token: string
    uid: number
    name: string
    isMan: boolean
    niangan: string
    nianzhi: string
    yuegan: string
    yuezhi: string
    rigan: string
    rizhi: string
    hourMode: number
    shigan: string
    shizhi: string
    ignoreError: boolean
}

export class LoadUserRequest {
    owner_id: number
}

export class DeleteUserRequest {
    owner_id: number
    user_id: number
}

export class LoginResult {
    code: number
    name: string
    token: string
    uid: number

    static build(responseText: string) {
        let result = new LoginResult()
        try {
            let obj = JSON.parse(responseText)
            result.code = obj["code"]
            let data = obj["data"]
            result.name = data["name"]
            result.token = data["token"]
            result.uid = data["uid"]
        } catch (e) {
            result.code = -1
        }
        return result
    }
}

export class SiZhuCheckResult {
    error: boolean
    adv_nian: string[]
    adv_yue: string[]
    adv_ri: string[]
    adv_shi: string[]
    adv_date: string[]

    static build(data: object) {
        let result = new SiZhuCheckResult()
        Object.keys(data).forEach(function (key) {
            let value = data[key]
            result[key] = value
        })
        return result
    }
}

export class UserInfo {
    id: number
    name: string
    isMan: boolean
    dateMode: number
    year: number
    month: string
    day: string
    hourMode: number
    hour: number
    minute: number
    areaMode: number
    sheng: string
    shi: string
    qu: string
    extra: string
    nongli_year: number
    nongli_month: string
    nongli_day: string
    req_nian_gan: string
    req_nian_zhi: string
    req_yue_gan: string
    req_yue_zhi: string
    req_ri_gan: string
    req_ri_zhi: string
    req_shi_gan: string
    req_shi_zhi: string

    rsp_nian_gan: string
    rsp_nian_zhi: string
    rsp_yue_gan: string
    rsp_yue_zhi: string
    rsp_ri_gan: string
    rsp_ri_zhi: string
    rsp_shi_gan: string
    rsp_shi_zhi: string
    owner_id: number

    getKey() {
        let key = ""
        key += this.id.toString()
        key += this.name
        key += this.isMan ? '男' : '女'
        if (this.dateMode == 0) {
            key += this.year.toString()
            key += this.month.toString()
            key += this.day.toString()
            if (this.hour != null) {
                key += this.hour.toString()
            }
            if (this.minute != null) {
                key += this.minute.toString()
            }
        } else if (this.dateMode == 1) {
            key += this.nongli_year.toString()
            key += this.nongli_month
            key += this.nongli_day
        } else if (this.dateMode == 2) {
            key += this.req_nian_gan
            key += this.req_nian_zhi
            key += this.req_yue_gan
            key += this.req_yue_zhi
            key += this.req_ri_gan
            key += this.req_ri_zhi
            if (this.req_shi_gan != null) {
                key += this.req_shi_gan
            } else {
                key += "无"
            }
            if (this.req_shi_zhi != null) {
                key += this.req_shi_zhi
            } else {
                key += "无"
            }
        }
        if (this.extra != null) {
            key += this.extra
        }
        return key
    }

    fillData(bazi: BaZi) {
        let req = bazi.base_info
        let info = this
        info.isMan = bazi.is_man
        info.rsp_nian_gan = bazi.nian_gan.text
        info.rsp_nian_zhi = bazi.nian_zhi.text
        info.rsp_yue_gan = bazi.yue_gan.text
        info.rsp_yue_zhi = bazi.yue_zhi.text
        info.rsp_ri_gan = bazi.ri_gan.text
        info.rsp_ri_zhi = bazi.ri_zhi.text
        info.rsp_shi_gan = bazi.shi_gan.text
        info.rsp_shi_zhi = bazi.shi_zhi.text
        info.year = req.year
        info.month = req.month?.toString()
        info.day = req.day?.toString()
        info.hourMode = req.hour_mode
        info.hour = req.hour
        info.minute = req.minute
        info.areaMode = req.area_mode
        info.sheng = req.sheng
        info.shi = req.shi
        info.qu = req.qu
        info.nongli_year = req.nongli_year
        info.nongli_month = req.nongli_month
        info.nongli_day = req.nongli_day
        info.req_nian_gan = req.nian_gan
        info.req_nian_zhi = req.nian_zhi
        info.req_yue_gan = req.yue_gan
        info.req_yue_zhi = req.yue_zhi
        info.req_ri_gan = req.ri_gan
        info.req_ri_zhi = req.ri_zhi
        info.req_shi_gan = req.shi_gan
        info.req_shi_zhi = req.shi_zhi
    }
}


export class GaoNeng {
    arg1: string
    arg2: string
    arg3: string
    wei_zhi1: number
    wei_zhi2: number
    wei_zhi3: number
    wu_xing1: string
    wu_xing2: string
    wu_xing3: string
    shi_shen1: string
    shi_shen2: string
    shi_shen3: string
    text: string
    guanxi: string
    leibie: string
    hehua: string
    fangxiang: string
    zengqiang_shishen: string
    tip: string
    sign: string
    xing_chu: string
    xing_chong: string
}

export class SiZhu {
    text: string
    wei_zhi: number
    wu_xing: string
    shi_shen: string
    yin_yang: string
    cang_gan1: string
    cang_gan2: string
    cang_gan3: string
    gan1_wu_xing: string
    gan2_wu_xing: string
    gan3_wu_xing: string
    gan1_shi_shen: string
    gan2_shi_shen: string
    gan3_shi_shen: string
    zhang_sheng_zhu: string
    zhang_sheng_ri: string
    dayun_text: string
    liunian_text: string
    liuyue_text: string
    liuri_text: string
    @use(Array, GaoNeng)
    gaonengs: GaoNeng[]
    is_muku: boolean
    is_muku_open: boolean
    @use(Array)
    shenshas: string[]
    @use(Map, Array, GaoNeng)
    trigger_gaonengs: Map<string, GaoNeng[]>
}

export class PowerCounter {
    @use(Map)
    power_raw: Map<string, number>
    @use(Map)
    power_muku: Map<string, number>
    @use(Map)
    power_gaoneng: Map<string, number>
    @use(Map)
    power_zihe: Map<string, number>
}


export class PowerDetail {
    raw: Map<string, number> = new Map()
    muku: Map<string, number> = new Map()
    gaoneng: Map<string, number> = new Map()
    zihe: Map<string, number> = new Map()

    rawWuXingTotal: number
    mukuWuXingTotal: number
    gaonengWuXingTotal: number
    ziheWuXingTotal: number

    rawShiShenTotal: number
    mukuShiShenTotal: number
    gaonengShiShenTotal: number
    ziheShiShenTotal: number

    powerMap: Map<number, any>
    wuxingTotals: Map<number, number>
    shishenTotals: Map<number, number>

    calc(steps: Map<string, number>, counter: PowerCounter) {
        let wuxingAry = ["木", "火", "土", "金", "水"]
        let shishenAry = ["官", "杀", "比", "劫", "食", "伤", "财", "才", "印", "枭"]
        this.rawWuXingTotal = 0
        this.mukuWuXingTotal = 0
        this.gaonengWuXingTotal = 0
        this.ziheWuXingTotal = 0

        this.rawShiShenTotal = 0
        this.mukuShiShenTotal = 0
        this.gaonengShiShenTotal = 0
        this.ziheShiShenTotal = 0

        let self = this
        wuxingAry.forEach(function (wx) {
            let step = steps.get(wx)
            let ctRaw = counter.power_raw.get(wx)
            let ctGaoNeng = counter.power_gaoneng.get(wx)
            let ctMuKu = counter.power_muku.get(wx)
            let ctZiHe = counter.power_zihe.get(wx)

            self.raw.set(wx, step * ctRaw)
            self.muku.set(wx, step * ctMuKu)
            self.gaoneng.set(wx, step * ctGaoNeng)
            self.zihe.set(wx, step * ctZiHe)

            self.rawWuXingTotal += step * ctRaw
            self.gaonengWuXingTotal += step * ctGaoNeng
            self.mukuWuXingTotal += step * ctMuKu
            self.ziheWuXingTotal += step * ctZiHe
        })

        shishenAry.forEach(function (ss) {
            let step = steps.get(ss)
            let ctRaw = counter.power_raw.get(ss)
            let ctGaoNeng = counter.power_gaoneng.get(ss)
            let ctMuKu = counter.power_muku.get(ss)
            let ctZiHe = counter.power_zihe.get(ss)

            self.raw.set(ss, step * ctRaw)
            self.muku.set(ss, step * ctMuKu)
            self.gaoneng.set(ss, step * ctGaoNeng)
            self.zihe.set(ss, step * ctZiHe)

            self.rawShiShenTotal += step * ctRaw
            self.gaonengShiShenTotal += step * ctGaoNeng
            self.mukuShiShenTotal += step * ctMuKu
            self.ziheShiShenTotal += step * ctZiHe
        })

        this.powerMap = new Map()
        this.powerMap.set(1, this.raw)
        this.powerMap.set(2, this.gaoneng)
        this.powerMap.set(4, this.muku)
        this.powerMap.set(8, this.zihe)

        this.wuxingTotals = new Map()
        this.wuxingTotals.set(1, this.rawWuXingTotal)
        this.wuxingTotals.set(2, this.gaonengWuXingTotal)
        this.wuxingTotals.set(4, this.mukuWuXingTotal)
        this.wuxingTotals.set(8, this.ziheWuXingTotal)

        this.shishenTotals = new Map()
        this.shishenTotals.set(1, this.rawShiShenTotal)
        this.shishenTotals.set(2, this.gaonengShiShenTotal)
        this.shishenTotals.set(4, this.mukuShiShenTotal)
        this.shishenTotals.set(8, this.ziheShiShenTotal)
    }

    private getValueByIndexs(indexs: number[], src: Map<any, any>, key: any) {
        let result = 0
        indexs.forEach(function (i) {
            let data = src.get(i)
            if (key != null) {
                result += data.get(key)
            } else {
                result += data
            }
        })
        return result
    }

    private getIndexs(mode: number) {
        let indexs = []
        switch (mode) {
            case 1:
                indexs = [1]
                break
            case 2:
                indexs = [2]
                break
            case 3:
                indexs = [1, 2]
                break
            case 4:
                indexs = [4]
                break
            case 5:
                indexs = [1, 4]
                break
            case 6:
                indexs = [2, 4]
                break
            case 7:
                indexs = [1, 2, 4]
                break
            case 8:
                indexs = [8]
                break
            case 9:
                indexs = [1, 8]
                break
            case 10:
                indexs = [2, 8]
                break
            case 11:
                indexs = [1, 2, 8]
                break
            case 12:
                indexs = [4, 8]
                break
            case 13:
                indexs = [1, 4, 8]
                break
            case 14:
                indexs = [2, 4, 8]
                break
            case 15:
                indexs = [1, 2, 4, 8]
                break
        }
        return indexs
    }

    getPower(key: string, mode: number = DataCenter.powerMode.value) {
        let value = 0
        if (key == null) {
            return value
        }

        let indexs = this.getIndexs(mode)
        value = this.getValueByIndexs(indexs, this.powerMap, key)
        return value
    }

    getWuXingTotal(mode: number = DataCenter.powerMode.value) {
        let indexs = this.getIndexs(mode)
        let value = this.getValueByIndexs(indexs, this.wuxingTotals, null)
        return value
    }

    /**
     * mode:1 => raw, 3=>raw+gaoneng 5=>raw+muku 7=>all
     * */
    getShiShenTotal(mode: number = DataCenter.powerMode.value) {
        let indexs = this.getIndexs(mode)
        let value = this.getValueByIndexs(indexs, this.shishenTotals, null)
        return value
    }
}

export class GeJuItem {
    arg1: string
    arg2: string
    shishen1: string
    shishen2: string
    guanxi: string
    guan_sha: number
    cai_xing: number
    yin_shou: number
    shi_shang: number
    bi_jie: number
    trigger: string

    constructor() {
        this.guan_sha = 0
        this.cai_xing = 0
        this.yin_shou = 0
        this.shi_shang = 0
        this.bi_jie = 0
    }

    getTotal() {
        return this.guan_sha + this.cai_xing + this.yin_shou + this.shi_shang + this.bi_jie
    }
}

export class GeJuScore {
    guan_sha: number = 0
    cai_xing: number = 0
    yin_shou: number = 0
    shi_shang: number = 0
    bi_jie: number = 0
    total: number = 0
}

export class SiZhuGeJu {
    yudao: string
    @use(Array, GeJuItem)
    items: GeJuItem[]
    guan_sha: number = 0
    cai_xing: number = 0
    yin_shou: number = 0
    shi_shang: number = 0
    bi_jie: number = 0
    total: number = 0

    filter(trigger: string) {
        let result = []
        this.items.forEach(function (item) {
            if (item.trigger == trigger) {
                result.push(item)
            }
        })
        return result
    }

    getScores(trigger: string) {
        let guanSha = 0
        let caiXing = 0
        let yinShou = 0
        let shiShang = 0
        let biJie = 0
        let all = 0
        this.items.forEach(function (item) {
            if (item.trigger == trigger) {
                guanSha += item.guan_sha
                caiXing += item.cai_xing
                yinShou += item.yin_shou
                shiShang += item.shi_shang
                biJie += item.bi_jie
            }
        })
        all = guanSha + caiXing + yinShou + shiShang + biJie
        return [guanSha, caiXing, yinShou, shiShang, biJie, all]

    }
}

export class XiongJi {
    name: string
    guanxi: string
    arg1: string
    arg2: string
    tip: string
    found: boolean
}

export class QuShiItem {
    score: number
    reasons: string[]

    constructor() {
        this.score = 0
        this.reasons = []
    }
}

export class LiuRi {
    day: number
    nong_li: string
    xing_qi: string
    gan_text: string
    zhi_text: string
    @use(SiZhu)
    gan_zhu: SiZhu
    @use(SiZhu)
    zhi_zhu: SiZhu
    is_gaoneng_calced: boolean = false
    trigger: string
}

export class LiuYue {
    month: number
    jie_qi: string
    jie_qi_date: string
    gan_text: string
    zhi_text: string
    @use(SiZhu)
    gan_zhu: SiZhu
    @use(SiZhu)
    zhi_zhu: SiZhu
    @use(Array, LiuRi)
    liuri: LiuRi[]
    trigger: string
    yinyuan_level: number = 0
    yinyuan_tip: string
    is_gaoneng_calced: boolean = false
    qushiItems: Map<string, QuShiItem> = new Map()

    findLiuRi(day: number) {
        let result = null
        for (let i = 0; i < this.liuri.length; i++) {
            let temp = this.liuri[i]
            if (temp.day == day) {
                result = temp
                break
            }
        }
        return result
    }
}

export class LiuNian {
    year: number
    gan_text: string
    zhi_text: string
    from_dayun: boolean
    age: number
    @use(SiZhu)
    gan_zhu: SiZhu
    @use(SiZhu)
    zhi_zhu: SiZhu
    @use(Array, LiuYue)
    liuyue: LiuYue[]
    trigger: string
    yinyuan_level: number = 0
    yinyuan_tip: string
    qushiItems: Map<string, QuShiItem> = new Map()

    findLiuYue(month: number) {
        let result = null
        for (let i = 0; i < this.liuyue.length; i++) {
            let temp = this.liuyue[i]
            if (temp.month == month) {
                result = temp
                break
            }
        }
        return result
    }
}

export class DaYunItem {
    is_dayun: boolean
    gan_text: string
    zhi_text: string
    trigger: string
    start_age: number
    end_age: number
    start_year: number
    end_year: number
    @use(SiZhu)
    gan_zhu: SiZhu
    @use(SiZhu)
    zhi_zhu: SiZhu
    @use(Array, LiuNian)
    liunian: LiuNian[]
    qushiItems: Map<string, QuShiItem> = new Map()

    findLiuNian(year: number) {
        let result = null
        for (let i = 0; i < this.liunian.length; i++) {
            let temp = this.liunian[i]
            if (temp.year == year) {
                result = temp
                break
            }
        }
        return result
    }
}

export class ZhenDuanItem {
    content: string
    order: number
    selected: boolean
    defaultSelected: boolean
    tip: string
    key: string
}

export class ZhenDuan {
    @use(Array, ZhenDuanItem)
    jianyi: ZhenDuanItem[]
    @use(Array, ZhenDuanItem)
    cankao: ZhenDuanItem[]

    getItems() {
        let lst = []
        let ary = [this.jianyi, this.cankao]
        ary.forEach(function (items) {
            items.forEach(function (item) {
                lst.push(item)
            })
        })
        return lst
    }
}

export class ZhenDuanSet {
    @use(ZhenDuan)
    wuxing: ZhenDuan
    @use(ZhenDuan)
    shishen: ZhenDuan
    @use(ZhenDuan)
    zuozhi: ZhenDuan
    @use(ZhenDuan)
    menhu: ZhenDuan
    @use(ZhenDuan)
    shensha: ZhenDuan
    @use(ZhenDuan)
    geju: ZhenDuan
    @use(Map, ZhenDuan)
    gaoneng: Map<string, ZhenDuan>
    @use(ZhenDuan)
    jiankang: ZhenDuan
    @use(ZhenDuan)
    qinggan: ZhenDuan
    @use(ZhenDuan)
    dayun: ZhenDuan
    @use(ZhenDuan)
    yinyuan: ZhenDuan

    getZhenDuanContent(target: ZhenDuan) {
        let s = ""
        let items = target.getItems()
        items.forEach(function (item) {
            if (item.selected) {
                s += item.content
            }
        })
        return s
    }

    getContent(includeQingGan: boolean, includeDaYun: boolean, includeYinYuan: boolean) {
        let self = this
        let s = ""
        let sWuXing = this.getZhenDuanContent(this.wuxing)
        if (sWuXing.length > 0) {
            s += "五行" + sWuXing + "\n\n"
        }
        let sShiShen = this.getZhenDuanContent(this.shishen)
        if (sShiShen.length > 0) {
            s += sShiShen + "\n\n"
        }
        let sZuoZhi = this.getZhenDuanContent(this.zuozhi)
        if (sZuoZhi.length > 0) {
            s += sZuoZhi + "\n\n"
        }
        let sMenHu = this.getZhenDuanContent(this.menhu)
        if (sMenHu.length > 0) {
            s += sMenHu + "\n\n"
        }
        let sShenSha = this.getZhenDuanContent(this.shensha)
        if (sShenSha.length > 0) {
            s += "神煞方面" + sShenSha + "\n\n"
        }
        let sGeJu = this.getZhenDuanContent(this.geju)
        if (sGeJu.length > 0) {
            s += "八字格局" + sGeJu + "\n\n"
        }
        let sGaoNeng = ""
        this.gaoneng.forEach(function (zd) {
            sGaoNeng += self.getZhenDuanContent(zd)
        })
        if (sGaoNeng.length > 0) {
            s += "八字原局" + sGaoNeng + "\n\n"
        }
        let sJianKang = this.getZhenDuanContent(this.jiankang)
        if (sJianKang.length > 0) {
            s += "健康方面" + sJianKang + "\n\n"
        }
        if (includeQingGan) {
            let sQingGan = this.getZhenDuanContent(this.qinggan)
            if (sQingGan.length > 0) {
                s += "情感方面" + sQingGan + "\n\n"
            }
        }
        if (includeDaYun) {
            let sDaYun = this.getZhenDuanContent(this.dayun)
            if (sDaYun.length > 0) {
                s += sDaYun + "\n\n"
            }
        }
        if (includeYinYuan) {
            let sYinYuan = this.getZhenDuanContent(this.yinyuan)
            if (sYinYuan.length > 0) {
                s += sYinYuan + "\n\n"
            }
        }

        return s
    }
}

export class BaseInfo {
    date_mode: number
    year: number
    month: number
    day: number
    hour_mode: number
    hour: number
    minute: number
    area_mode: number
    sheng: string
    shi: string
    qu: string
    nongli_year: number
    nongli_month: string
    nongli_day: string
    nian_gan: string
    nian_zhi: string
    yue_gan: string
    yue_zhi: string
    ri_gan: string
    ri_zhi: string
    shi_gan: string
    shi_zhi: string
}


export class DailyQuShi {
    ri: Map<string, number> = new Map()
    shi: Map<string, number> = new Map()
    total: Map<string, number> = new Map()
    detailsRi: Map<string, string[]> = new Map()
    detailsShi: Map<string, string[]> = new Map()

    getValue(key: string, mode: number) {
        if (mode == 0) {
            if (this.ri.has(key)) {
                return this.ri.get(key)
            } else {
                this.ri.set(key, 0)
                return 0
            }
        } else if (mode == 1) {
            if (this.shi.has(key)) {
                return this.shi.get(key)
            } else {
                this.shi.set(key, 0)
                return 0
            }
        }
    }

    setValue(key: string, value: number, mode: number) {
        if (mode == 0) {
            this.ri.set(key, value)
        } else if (mode == 1) {
            this.shi.set(key, value)
        }
    }

    addDetail(key: string, s: string, mode: number) {
        let lst = this.detailsRi
        if (mode == 1) {
            lst = this.detailsShi
        }
        if (!lst.has(key)) {
            lst.set(key, [])
        }
        let temp = lst.get(key)
        temp.push(s)
    }

    calcTotal() {
        let shishen_list = ["官", "杀", "财", "才", "印", "枭", "食", "伤", "比", "劫"]
        let self = this
        shishen_list.forEach(function (ss) {
            let temp = 0
            if (self.ri.has(ss)) {
                temp += self.ri.get(ss)
            }
            //注意，这里出来的月数值已经是累加过后的，是基于上个月的月数据再累加上去的。
            for (let i = 9; i <= 21; i += 2) {
                let key = ss + "_" + i.toString()
                if (self.shi.has(key)) {
                    temp += self.shi.get(key)
                }
                self.total.set(key, temp)
            }
        })
    }
}

export class QuShiInfo {
    year: number
    yun: Map<string, number> = new Map()
    nian: Map<string, number> = new Map()
    yue: Map<string, number> = new Map()
    total: Map<string, number> = new Map()
    detailsYun: Map<string, string[]> = new Map()
    detailsNian: Map<string, string[]> = new Map()
    detailsYue: Map<string, string[]> = new Map()

    getValue(key: string, mode: number) {
        if (mode == 0) {
            if (this.yun.has(key)) {
                return this.yun.get(key)
            } else {
                this.yun.set(key, 0)
                return 0
            }
        } else if (mode == 1) {
            if (this.nian.has(key)) {
                return this.nian.get(key)
            } else {
                this.nian.set(key, 0)
                return 0
            }
        } else if (mode == 2) {
            if (this.yue.has(key)) {
                return this.yue.get(key)
            } else {
                this.yue.set(key, 0)
                return 0
            }
        }
    }

    setValue(key: string, value: number, mode: number) {
        if (mode == 0) {
            this.yun.set(key, value)
        } else if (mode == 1) {
            this.nian.set(key, value)
        } else if (mode == 2) {
            this.yue.set(key, value)
        }
    }

    addDetail(key: string, s: string, mode: number) {
        let lst = this.detailsYun
        if (mode == 1) {
            lst = this.detailsNian
        } else if (mode == 2) {
            lst = this.detailsYue
        }
        if (!lst.has(key)) {
            lst.set(key, [])
        }
        let temp = lst.get(key)
        temp.push(s)
    }

    calcTotal() {
        let shishen_list = ["官", "杀", "财", "才", "印", "枭", "食", "伤", "比", "劫"]
        let self = this
        shishen_list.forEach(function (ss) {
            let temp = 0
            if (self.yun.has(ss)) {
                temp += self.yun.get(ss)
            }
            if (self.nian.has(ss)) {
                temp += self.nian.get(ss)
            }
            //注意，这里出来的月数值已经是累加过后的，是基于上个月的月数据再累加上去的。
            for (let i = 1; i <= 12; i++) {
                let key = ss + "_" + i.toString()
                if (self.yue.has(key)) {
                    temp += self.yue.get(key)
                }
                self.total.set(key, temp)
            }
        })
    }
}

export class RiYunShi {
    liu_nian_gan: SiZhu
    liu_nian_zhi: SiZhu
    liu_yue_gan: SiZhu
    liu_yue_zhi: SiZhu
    liu_ri_gan: SiZhu
    liu_ri_zhi: SiZhu
    ri_gan: SiZhu
}

export class BaZi {
    is_man: boolean
    tai_yang_shi: string
    @use(SiZhu)
    nian_gan: SiZhu
    @use(SiZhu)
    nian_zhi: SiZhu
    @use(SiZhu)
    yue_gan: SiZhu
    @use(SiZhu)
    yue_zhi: SiZhu
    @use(SiZhu)
    ri_gan: SiZhu
    @use(SiZhu)
    ri_zhi: SiZhu
    @use(SiZhu)
    shi_gan: SiZhu
    @use(SiZhu)
    shi_zhi: SiZhu
    tong_dang: number
    yi_dang: number
    @use(Map)
    power_steps: Map<string, number>
    @use(PowerCounter)
    counter: PowerCounter
    @use(Map)
    shishen_guanxi: Map<string, string[]>
    @use(Map, SiZhuGeJu)
    tiangan_geju: Map<string, SiZhuGeJu>
    @use(Map, SiZhuGeJu)
    dizhi_geju: Map<string, SiZhuGeJu>
    @use(Map)
    geju_score: Map<string, number>
    @use(Map, Array, XiongJi)
    geju_xiongji: Map<string, XiongJi[]>
    @use(Array, DaYunItem)
    dayun: DaYunItem[]
    @use(Map, SiZhuGeJu)
    dayun_tiangan_geju: Map<string, SiZhuGeJu>
    @use(Map, SiZhuGeJu)
    dayun_dizhi_geju: Map<string, SiZhuGeJu>
    @use(Map, Map)
    dayun_geju_score: Map<string, Map<string, number>>
    @use(Map, SiZhuGeJu)
    liunian_tiangan_geju: Map<string, SiZhuGeJu>
    @use(Map, SiZhuGeJu)
    liunian_dizhi_geju: Map<string, SiZhuGeJu>
    @use(Map, Map)
    liunian_geju_score: Map<string, Map<string, number>>
    @use(ZhenDuanSet)
    zhenduan: ZhenDuanSet
    @use(BaseInfo)
    base_info: BaseInfo


    //额外的
    powerDetail: PowerDetail
    qushiInfo: QuShiInfo[]
    dailyQuShi: Map<string, DailyQuShi>[]
    dailyQuShiDate: string[]


    static build(obj: object) {
        let bazi = new BaZi()
        resolve(bazi, obj)
        console.log("bazi:", bazi)
        bazi.powerDetail = new PowerDetail()
        bazi.powerDetail.calc(bazi.power_steps, bazi.counter)
        return bazi
    }

    allSiZhu() {
        return [this.nian_gan, this.nian_zhi, this.yue_gan, this.yue_zhi, this.ri_gan, this.ri_zhi, this.shi_gan, this.shi_zhi]
    }
}